@@ -3,7 +3,7 @@
 
 from trytond.model import fields
 from trytond.pool import Pool, PoolMeta
-from trytond.pyson import If, Eval, Bool
+from trytond.pyson import Bool, Eval, If
 
 
 def AccountTypeMixin(template=False):
@@ -19,8 +19,7 @@ def AccountTypeMixin(template=False):
             states={
                 'invisible': ((Eval('statement') != 'balance')
                     | Eval('assets', True)),
-                },
-            depends=['statement', 'assets'])
+                })
     if not template:
         for fname in dir(Mixin):
             field = getattr(Mixin, fname)
@@ -71,6 +70,10 @@ class Invoice(metaclass=PoolMeta):
 class InvoiceLine(metaclass=PoolMeta):
     __name__ = 'account.invoice.line'
 
+    advance_payment_recalled_lines = fields.One2Many(
+        'account.invoice.line', 'origin', "Advance Payment Recalled Lines",
+        readonly=True)
+
     @classmethod
     def _account_domain(cls, type_):
         domain = super()._account_domain(type_)
@@ -78,7 +81,22 @@ class InvoiceLine(metaclass=PoolMeta):
             domain.append(('type.unearned_revenue', '=', True))
         return domain
 
+    @property
+    def origin_name(self):
+        pool = Pool()
+        Condition = pool.get('sale.advance_payment.condition')
+        name = super().origin_name
+        if isinstance(self.origin, Condition) and self.origin.id >= 0:
+            name = self.origin.sale.rec_name
+        return name
+
     @classmethod
     def _get_origin(cls):
         return (super(InvoiceLine, cls)._get_origin()
             + ['sale.advance_payment.condition'])
+
+    @classmethod
+    def copy(cls, lines, default=None):
+        default = default.copy() if default is not None else {}
+        default.setdefault('advance_payment_recalled_lines')
+        return super().copy(lines, default=default)
