@@ -1,18 +1,20 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
+from itertools import chain
+
 from simpleeval import simple_eval
-from itertools import chain, groupby
 
 from trytond.i18n import gettext
-from trytond.pool import Pool, PoolMeta
 from trytond.model import (
-    ModelSQL, ModelView, Workflow, DeactivableMixin, fields)
-from trytond.pyson import Eval, Bool
-from trytond.tools import decistmt
-from trytond.transaction import Transaction
+    DeactivableMixin, ModelSQL, ModelView, Workflow, fields)
 from trytond.modules.company.model import (
     CompanyMultiValueMixin, CompanyValueMixin)
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Bool, Eval
+from trytond.tools import decistmt
+from trytond.transaction import Transaction
 
 from .exceptions import FormulaError
 
@@ -49,7 +51,7 @@ class AdvancePaymentTermLine(ModelView, ModelSQL, CompanyMultiValueMixin):
 
     advance_payment_term = fields.Many2One(
         'sale.advance_payment_term', "Advance Payment Term",
-        required=True, ondelete='CASCADE', select=True)
+        required=True, ondelete='CASCADE')
     description = fields.Char(
         "Description", required=True, translate=True,
         help="Used as description for the invoice line.")
@@ -127,15 +129,14 @@ class AdvancePaymentTermLineAccount(ModelSQL, CompanyValueMixin):
     __name__ = 'sale.advance_payment_term.line.account'
 
     line = fields.Many2One(
-        'sale.advance_payment_term.line', 'Line', required=True, select=True,
-        ondelete='CASCADE')
+        'sale.advance_payment_term.line', "Line",
+        required=True, ondelete='CASCADE')
     account = fields.Many2One(
         'account.account', "Account", required=True,
         domain=[
             ('type.unearned_revenue', '=', True),
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
 
 
 class AdvancePaymentCondition(ModelSQL, ModelView):
@@ -146,35 +147,26 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
     _states = {
         'readonly': Eval('sale_state') != 'draft',
         }
-    _depends = ['sale_state']
 
-    sale = fields.Many2One('sale.sale', 'Sale', required=True,
-        ondelete='CASCADE', select=True,
+    sale = fields.Many2One(
+        'sale.sale', "Sale", required=True, ondelete='CASCADE',
         states={
             'readonly': ((Eval('sale_state') != 'draft')
                 & Bool(Eval('sale'))),
-            },
-        depends=['sale_state'])
-    description = fields.Char(
-        "Description", required=True, states=_states, depends=_depends)
-    amount = fields.Numeric(
-        "Amount",
-        digits=(16, Eval('_parent_sale', {}).get('currency_digits', 2)),
-        states=_states, depends=_depends)
+            })
+    description = fields.Char("Description", required=True, states=_states)
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', states=_states)
     account = fields.Many2One(
         'account.account', "Account", required=True,
         domain=[
             ('type.unearned_revenue', '=', True),
             ('company', '=', Eval('sale_company')),
             ],
-        states=_states,
-        depends=_depends + ['sale_company'])
-    block_supply = fields.Boolean(
-        "Block Supply", states=_states, depends=_depends)
-    block_shipping = fields.Boolean(
-        "Block Shipping", states=_states, depends=_depends)
-    invoice_delay = fields.TimeDelta(
-        "Invoice Delay", states=_states, depends=_depends)
+        states=_states)
+    block_supply = fields.Boolean("Block Supply", states=_states)
+    block_shipping = fields.Boolean("Block Shipping", states=_states)
+    invoice_delay = fields.TimeDelta("Invoice Delay", states=_states)
 
     invoice_lines = fields.One2Many(
         'account.invoice.line', 'origin', "Invoice Lines", readonly=True)
@@ -184,9 +176,11 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
             'get_sale_states', "Sale State"), 'on_change_with_sale_state')
     sale_company = fields.Function(fields.Many2One(
             'company.company', "Company"), 'on_change_with_sale_company')
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"),
+        'on_change_with_currency')
 
     del _states
-    del _depends
 
     @classmethod
     def __setup__(cls):
@@ -206,8 +200,11 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
 
     @fields.depends('sale', '_parent_sale.company')
     def on_change_with_sale_company(self, name=None):
-        if self.sale and self.sale.company:
-            return self.sale.company.id
+        return self.sale.company if self.sale else None
+
+    @fields.depends('sale', '_parent_sale.currency')
+    def on_change_with_currency(self, name=None):
+        return self.sale.currency if self.sale else None
 
     @classmethod
     def copy(cls, conditions, default=None):
@@ -230,8 +227,6 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
             return None
         invoice.lines = invoice_lines
         invoice.save()
-
-        invoice.update_taxes()
         return invoice
 
     def get_invoice_advance_payment_lines(self, invoice):
@@ -283,15 +278,13 @@ class Sale(metaclass=PoolMeta):
         'Advance Payment Term',
         ondelete='RESTRICT', states={
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
+            })
     advance_payment_conditions = fields.One2Many(
         'sale.advance_payment.condition', 'sale',
         'Advance Payment Conditions',
         states={
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
+            })
     advance_payment_invoices = fields.Function(fields.Many2Many(
             'account.invoice', None, None, "Advance Payment Invoices"),
         'get_advance_payment_invoices',
@@ -346,48 +339,26 @@ class Sale(metaclass=PoolMeta):
     @classmethod
     def search_advance_payment_invoices(cls, name, clause):
         return [('advance_payment_conditions.invoice_lines.invoice'
-                + clause[0].lstrip(name),)
-            + tuple(clause[1:])]
+                + clause[0][len(name):], *clause[1:])]
 
-    def get_invoice_state(self):
-        state = super(Sale, self).get_invoice_state()
-        skip_ids = set(x.id for x in self.invoices_ignored)
-        skip_ids.update(x.id for x in self.invoices_recreated)
-        invoices = [i
-            for i in self.advance_payment_invoices if i.id not in skip_ids]
-        if invoices:
-            if any(i.state == 'cancelled' for i in invoices):
-                return 'exception'
-            elif all(i.state == 'paid' for i in invoices):
-                return state
-            else:
-                return 'waiting'
-        return state
+    @property
+    def _invoices_for_state(self):
+        return super()._invoices_for_state + self.advance_payment_invoices
 
     def get_recall_lines(self, invoice):
         pool = Pool()
         InvoiceLine = pool.get('account.invoice.line')
 
         recall_lines = []
-        advance_lines = InvoiceLine.search([
-                ('type', '=', 'line'),
-                ('origin', 'in', [str(c)
-                        for c in self.advance_payment_conditions]),
-                ('invoice.state', '=', 'paid'),
-                ])
-        recalled_lines = InvoiceLine.search([
-                ('type', '=', 'line'),
-                ('origin', 'in', [str(al) for al in advance_lines]),
-                ('invoice.state', '!=', 'cancelled'),
-                ],
-            order=[('origin', None)])
-        recalled_lines = {
-            k: list(v) for k, v in groupby(recalled_lines, lambda l: l.origin)}
+        advance_lines = [
+            l
+            for c in self.advance_payment_conditions
+            for l in c.invoice_lines
+            if l.type == 'line' and l.invoice.state == 'paid']
         for advance_line in advance_lines:
             amount = advance_line.amount
-            if advance_line in recalled_lines:
-                for recalled_line in recalled_lines[advance_line]:
-                    amount += recalled_line.amount
+            for recalled_line in advance_line.advance_payment_recalled_lines:
+                amount += recalled_line.amount
             if amount:
                 line = InvoiceLine(
                     invoice=invoice,
@@ -470,8 +441,8 @@ class SaleLine(metaclass=PoolMeta):
             return None
         return move
 
-    def get_purchase_request(self):
-        request = super(SaleLine, self).get_purchase_request()
+    def get_purchase_request(self, product_quantities):
+        request = super().get_purchase_request(product_quantities)
         if (self.sale.advance_payment_eligible()
                 and self.sale.supply_blocked):
             return None
