@@ -3,7 +3,6 @@
 from datetime import timedelta
 
 from proteus import Model
-
 from trytond.modules.company.tests.tools import get_company
 
 
@@ -30,7 +29,7 @@ def add_advance_payment_accounts(accounts, company=None, config=None):
     Account = Model.get('account.account', config=config)
 
     if not company:
-        company = get_company()
+        company = get_company(config=config)
 
     accounts['advance_payment'], = Account.find([
             ('type.unearned_revenue', '=', True),
